@@ -133,6 +133,62 @@ class CurrentReading:
         return value * multiplier
 
 
+@dataclass
+class Version:
+    major: int = -1
+    minor: int = -1
+    patch: int = -1
+
+    def __init__(self, major, minor, patch):
+        self.major = major
+        self.minor = minor
+        self.patch = patch
+
+    def __str__(self):
+        return f"v{self.major}.{self.minor}.{self.patch}"
+
+
+class CalibrationState(IntEnum):
+    """Enum for calibration state"""
+    NOT_ACTIVE = 0
+    END_REQUEST = 1
+    IN_PROGRESS = 2
+    ERROR = 3
+
+
+@dataclass
+class ManufacturerData:
+    """dataclass to store manufacturer data"""
+
+    disconnected: bool = False
+    calibration_state: CalibrationState  = -1
+    dfu_active: bool = False
+    integrations: bool = False
+    version: Version = None
+
+    def decode(self, value: tuple):
+        self.disconnected = self._get_b(value[0], 0)
+        self.calibration_state = CalibrationState(self._get_uint2(value[0], 2))
+        self.dfu_active = self._get_b(value[0], 4)
+        self.integrations = self._get_b(value[0], 5)
+        self.version = Version(value[3], value[2], value[1])
+
+    def _get_b(self, value, pos):
+        return True if value & (1 << pos) else False
+
+    def _get_uint2(self, value, pos):
+        return (value >> pos) & 0x03
+
+
+@dataclass
+class Aranet4Advertisement:
+    """dataclass to store the information aboud scanned aranet4 device"""
+
+    device: BLEDevice = None
+    readings: CurrentReading = None
+    manufacturer_data: ManufacturerData = None
+
+
 @dataclass
 class RecordItem:
     """dataclass to store historical records"""
@@ -183,9 +239,13 @@ class Aranet4:
     # Param return value if no data
     AR4_NO_DATA_FOR_PARAM = -1
 
+    # Manufacurer id
+    MANUFACTURER_ID = 0x0702
+
     # Aranet UUIDs and handles
     # Services
-    AR4_SERVICE = "f0cd1400-95da-4f4b-9ac8-aa55d312af0c"
+    AR4_OLD_SERVICE = "f0cd1400-95da-4f4b-9ac8-aa55d312af0c" # until v1.2.0
+    AR4_SERVICE = "0000fce0-0000-1000-8000-00805f9b34fb" # v1.2.0 and later
     GENERIC_SERVICE = "00001800-0000-1000-8000-00805f9b34fb"
     COMMON_SERVICE = "0000180a-0000-1000-8000-00805f9b34fb"
 
@@ -206,7 +266,8 @@ class Aranet4:
     COMMON_READ_MODEL_NUMBER = "00002a24-0000-1000-8000-00805f9b34fb"
     COMMON_READ_SERIAL_NO = "00002a25-0000-1000-8000-00805f9b34fb"
     COMMON_READ_HW_REV = "00002a27-0000-1000-8000-00805f9b34fb"
-    COMMON_READ_SW_REV = "00002a28-0000-1000-8000-00805f9b34fb"
+    COMMON_READ_FACTORY_SW_REV = "00002a28-0000-1000-8000-00805f9b34fb"
+    COMMON_READ_SW_REV = "00002a26-0000-1000-8000-00805f9b34fb"
     COMMON_READ_BATTERY = "00002a19-0000-1000-8000-00805f9b34fb"
 
     # Write / Aranet service
@@ -283,7 +344,7 @@ class Aranet4:
     async def get_last_measurement_date(self, use_epoch: bool = False):
         """Calculate the time the last datapoint was logged"""
         ago = await self.get_seconds_since_update()
-        now = datetime.datetime.utcnow().replace(microsecond=0)
+        now = datetime.datetime.now(datetime.timezone.utc).replace(microsecond=0)
         delta_ago = datetime.timedelta(seconds=ago)
         last_reading = now - delta_ago
         if use_epoch:
@@ -418,19 +479,19 @@ def _calc_start_end(datapoint_times: int, entry_filter):
     if filter_start:
         time_start = -1
         for idx, timestamp in enumerate(datapoint_times, start=1):
-            if filter_start < timestamp:
+            if filter_start <= timestamp:
                 time_start = idx
                 break
-        if 0 < time_start < end:
+        if 0 < time_start <= end:
             start = time_start
     if filter_end:
         time_end = -1
         for idx, timestamp in enumerate(datapoint_times, start=1):
-            if timestamp < filter_end:
+            if timestamp <= filter_end:
                 time_end = idx
             else:
                 break
-        if start < time_end < end:
+        if start <= time_end <= end:
             end = time_end
     return start, end
 
@@ -451,22 +512,70 @@ def get_current_readings(mac_address: str) -> CurrentReading:
     return asyncio.run(_current_reading(mac_address))
 
 
-async def _find_nearby(detection_callback: callable,
-                       duration: int = 5) -> List[BLEDevice]:
-    scanner = BleakScanner()
-    scanner.register_detection_callback(detection_callback)
-    print("Looking for Aranet4 devices...")
+class Aranet4Scanner:
+    """Aranet4 Scanner class - scan advertisements and process data, if available"""
+
+    def _process_advertisement(self, device, ad_data):
+        """Processes Aranet4 advertisement data"""
+
+        adv = Aranet4Advertisement()
+        adv.device = device
+
+        has_manufacurer_data = Aranet4.MANUFACTURER_ID in ad_data.manufacturer_data
+        has_service = Aranet4.AR4_SERVICE in ad_data.service_data
+
+        if has_manufacurer_data:
+            mf_data = ManufacturerData()
+            raw_bytes = ad_data.manufacturer_data[Aranet4.MANUFACTURER_ID]
+
+            # Basic info
+            value_fmt = "<BBBHBB"
+            value = struct.unpack(value_fmt, raw_bytes[0:7])
+            mf_data.decode(value)
+            adv.manufacturer_data = mf_data
+
+            # Extended info / measurements
+            if len(raw_bytes) >= 20:
+                value_fmt = "<HHHBBBHH"
+                value = struct.unpack(value_fmt, raw_bytes[8:21])
+                adv.readings = CurrentReading()
+                adv.readings.decode(value)
+                adv.readings.name = device.name
+
+
+        self.on_scan(adv)
+
+    def __init__(self, on_scan):
+        uuids = [Aranet4.AR4_SERVICE, Aranet4.AR4_OLD_SERVICE]
+        self.on_scan = on_scan
+        self.scanner = BleakScanner(
+            detection_callback=self._process_advertisement,
+            service_uuids=uuids
+        )
+
+    async def start(self):
+        await self.scanner.start()
+
+    async def stop(self):
+        await self.scanner.stop()
+
+async def _find_nearby(detect_callback: callable, duration: int) -> List[BLEDevice]:
+    scanner = Aranet4Scanner(detect_callback)
     await scanner.start()
     await asyncio.sleep(duration)
     await scanner.stop()
     return [device
-            for device in scanner.discovered_devices
+            for device in scanner.scanner.discovered_devices
             if "Aranet4" in device.name]
 
 
-def find_nearby(detect_callback: callable) -> List[BLEDevice]:
-    return asyncio.run(_find_nearby(detect_callback))
+def find_nearby(detect_callback: callable, duration: int = 5) -> List[BLEDevice]:
+    """
+    Scans for nearby Aranet4 devices.
+    Will call callback on every valid Aranet4 advertisement, including duplcates
+    """
 
+    return asyncio.run(_find_nearby(detect_callback, duration))
 
 async def _all_records(address, entry_filter):
     """
@@ -487,7 +596,7 @@ async def _all_records(address, entry_filter):
     dev_name = await monitor.get_name()
     dev_version = await monitor.get_version()
     last_log = await monitor.get_seconds_since_update()
-    now = datetime.datetime.utcnow().replace(microsecond=0)
+    now = datetime.datetime.now(datetime.timezone.utc).replace(microsecond=0)
     interval = await monitor.get_interval()
     next_log = interval - last_log
     # Decide if there is enough time to read all the data
@@ -498,7 +607,7 @@ async def _all_records(address, entry_filter):
         await asyncio.sleep(next_log)
         # there was another log so update the numbers
         last_log = await monitor.get_seconds_since_update()
-        now = datetime.datetime.utcnow().replace(microsecond=0)
+        now = datetime.datetime.now(datetime.timezone.utc).replace(microsecond=0)
 
     log_size = await monitor.get_total_readings()
     log_points = _log_times(now, log_size, interval, last_log)
